%
% input:
%    - g = degraded image
%    - h = psf
%    - method = 1 - Wiener, 2 - Constrained Least Squares
%    - param = noise variance
%    - r = real image
%
% output
%    - f = restored image
%    - e = restoration error (mean squared error)
%
function [f,e] = LSF(g, h, method, param, r)

  H = fftn(h, size(g));
  G = fftn(g);
  W = H; % filter
  [N,M] = size(G);
  if (method == 1) % Wiener
     disp('Wiener Filter');
     Sn = repmat(param, size(G)); % noise power spectrum
     Sf = ((G*G')/(N*M))-param; % image power spectrum
     W = (H' .* Sf) ./ ((abs(H).^2) .* Sf + Sn); % Wiener filter
  else
     if (method == 2) % Constrained Least Squares
	disp('Constrained Least-Squares Filter');
	p = [0 -1 0; -1 4 -1; 0 -1 0]; % laplacian operator
	P = fftn(p, size(g));
	W = H' ./ ((abs(H).^2) + param*(abs(P).^2)); % build CLS filter
     end
  end
  f = real(ifftn(G.*W)); % perform filtering
  e = sqrt(sum(sum( (f-r).^2 )));         

